@@ -1,38 +1,28 @@
-(* Otags reloaded
+(* Otags III
  * 
  * Hendrik Tews Copyright (C) 2010 - 2016
  * 
- * This file is part of "Otags reloaded".
+ * This file is part of "Otags III".
  * 
- * "Otags reloaded" is free software: you can redistribute it and/or
+ * "Otags III" is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
  * 
- * "Otags reloaded" is distributed in the hope that it will be useful,
+ * "Otags III" is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License in file COPYING in this or one of the parent
  * directories for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with "Otags reloaded". If not, see
+ * along with "Otags III". If not, see
  * <http://www.gnu.org/licenses/>.
  * 
- * $Id: conf.mli,v 1.5 2016/01/10 20:57:39 tews Exp $
- * 
  * configuration
  * 
  *)
 
-val camlp4_exec_path : string
-
-val camlp4_mod_path : string
-
 val ocaml_version : string
 
 val otags_version : string
-
-val otags_camlp4_ast_intf_magic : string
-
-val otags_camlp4_ast_impl_magic : string
