@@ -1,36 +1,33 @@
-(* Otags reloaded
+(* Otags III
  * 
  * Hendrik Tews Copyright (C) 2010 - 2016
  * 
- * This file is part of "Otags reloaded".
+ * This file is part of "Otags III".
  * 
- * "Otags reloaded" is free software: you can redistribute it and/or
+ * "Otags III" is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation, either version 3 of the
  * License, or (at your option) any later version.
  * 
- * "Otags reloaded" is distributed in the hope that it will be useful,
+ * "Otags III" is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License in file COPYING in this or one of the parent
  * directories for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with "Otags reloaded". If not, see
+ * along with "Otags III". If not, see
  * <http://www.gnu.org/licenses/>.
  * 
- * $Id: source_channel.mli,v 1.4 2016/01/10 20:57:40 tews Exp $
- * 
  * hold the in_channel of the source file
  * 
  *)
 
 
-open Types
-
 val reset : unit -> unit
 
-val get_channel : ?primary_file:bool -> loc_t -> in_channel
+val get_channel : ?primary_file:bool -> Location.t -> in_channel
     
 
-val full_string_of_loc : Camlp4.PreCast.Loc.t -> string
+(* XXX needed? *)
+val full_string_of_loc : Location.t -> string
