@@ -21,6 +21,16 @@
  * 
  * recursive tagging function
  * 
+ * This file depends on the OCaml syntax tree and is therefore likely
+ * to break with each new OCaml version. To make it a bit more robust
+ * all pattern matches contain catch-all clauses such that new syntax
+ * pieces are ignored (instead of crashing otags). In order to
+ * conveniently find missing new syntax, otags should be test build
+ * with ``make devel-build'', which compiles this file with the
+ * ppx_flush_catchall preprocessor and with -w A (instead of -w
+ * A-4-11). ppx_flush_catchall deletes in each pattern match that is
+ * annotated with the @flushcatchall attribute the last catch-all
+ * match ``| _ -> ()'', possibly inside an or pattern.
  *)
 
 open Otags_misc
@@ -37,7 +47,7 @@ open Asttypes
 
 let tag_type_decl write_tag {ptype_name = {txt; loc} ; ptype_kind; _ } =
   write_tag loc txt;
-  match ptype_kind with
+  match [@flushcatchall] ptype_kind with
     | Ptype_variant(constr_decl_list) ->
         (* Invariant: non-empty list *)
        List.iter
@@ -48,41 +58,44 @@ let tag_type_decl write_tag {ptype_name = {txt; loc} ; ptype_kind; _ } =
        List.iter
 	 (fun {pld_name = {txt; loc}; _} -> write_tag loc txt)
 	 field_decl_list
-  | Ptype_abstract
-  | Ptype_open ->
-     (* XXX: .. (extensible variant type) *)
-     ()
+    | Ptype_abstract
+    | Ptype_open
+       (* XXX: .. (extensible variant type) *)
+    | _ -> ()
 
 let tag_extension_constuctor write_tag {pext_name = {txt; loc}; _} =
   write_tag loc txt
 
-let rec tag_class_field_decl write_tag ct = match ct.pctf_desc with
-  | Pctf_inherit ct ->
-        (* inherit CT *)
-     (* The class type can be a concrete class body 
-      * type object ... end, therefore we tag it.
-      *)
-     tag_class_type write_tag ct
-  (* XXX missing locations for attributes and methods
-   * | Pctf_val of (string * mutable_flag * virtual_flag * core_type)
-   *       (\* val x: T *\)
-   * | Pctf_method  of (string * private_flag * virtual_flag * core_type)
-   *       (\* method x: T
-   * 
-   *          Note: T can be a Ptyp_poly.
-   *        *\)
-   *)
-  | Pctf_val _
-  | Pctf_method _
-  | Pctf_constraint _
-        (* constraint T1 = T2 *)
-  | Pctf_attribute _
-        (* [@@@id] *)
-  | Pctf_extension _
-        (* [%%id] *)
-    -> ()
+let rec tag_class_field_decl write_tag ct =
+  match [@flushcatchall] ct.pctf_desc with
+    | Pctf_inherit ct ->
+          (* inherit CT *)
+       (* The class type can be a concrete class body 
+	* type object ... end, therefore we tag it.
+	*)
+       tag_class_type write_tag ct
+    (* XXX missing locations for attributes and methods
+     * | Pctf_val of (string * mutable_flag * virtual_flag * core_type)
+     *       (\* val x: T *\)
+     * | Pctf_method  of (string * private_flag * virtual_flag * core_type)
+     *       (\* method x: T
+     * 
+     *          Note: T can be a Ptyp_poly.
+     *        *\)
+     *)
+    (* 
+     * | Pctf_val _
+     * | Pctf_method _
+     *)
+    | Pctf_constraint _
+	  (* constraint T1 = T2 *)
+    | Pctf_attribute _
+	  (* [@@@id] *)
+    | Pctf_extension _
+	  (* [%%id] *)
+    | _ -> ()
 
-and tag_class_type write_tag ct = match ct.pcty_desc with
+and tag_class_type write_tag ct = match [@flushcatchall] ct.pcty_desc with
   | Pcty_signature {pcsig_fields; _ } ->
         (* object ... end *)
      List.iter (tag_class_field_decl write_tag) pcsig_fields
@@ -99,13 +112,15 @@ and tag_class_type write_tag ct = match ct.pcty_desc with
   | Pcty_extension _
         (* [%id] *)
     -> ()
+  | _ -> ()
 
 let tag_class_decl write_tag {pci_name = {txt; loc}; pci_expr; _} =
     write_tag loc txt;
     tag_class_type write_tag pci_expr
 
 
-let rec tag_sig_item write_tag sig_item = match sig_item.psig_desc with
+let rec tag_sig_item write_tag sig_item =
+  match [@flushcatchall] sig_item.psig_desc with
     | Psig_value({pval_name = {txt; loc}; _}) -> write_tag loc txt
 	  (*
 	    val x: T
@@ -146,12 +161,13 @@ let rec tag_sig_item write_tag sig_item = match sig_item.psig_desc with
     	  (* [@@@id] *)
     | Psig_extension _
     	  (* [%%id] *)
-      -> ()
+    | _ -> ()
+	     
 and tag_module_declaration write_tag {pmd_name = {txt; loc}; pmd_type; _ } =
   write_tag loc txt;
   tag_module_type write_tag pmd_type
 
-and tag_module_type write_tag mt = match mt.pmty_desc with
+and tag_module_type write_tag mt = match [@flushcatchall] mt.pmty_desc with
   | Pmty_ident _ -> ()
         (* S *)
     (* The id here can be a long one with dots and applications.
@@ -175,42 +191,43 @@ and tag_module_type write_tag mt = match mt.pmty_desc with
         (* [%id] *)
   | Pmty_alias _
         (* (module M) *)
-    -> ()
+  | _ -> ()
 	 
 and tag_module_type_decl write_tag {pmtd_name = {txt; loc}; pmtd_type; _ } =
   write_tag loc txt;
   option_map (tag_module_type write_tag) pmtd_type
 
-let rec tag_class_field_def write_tag cfd = match cfd.pcf_desc with
-  | Pcf_inherit(_, ce, _) ->
-        (* inherit CE
-           inherit CE as x
-           inherit! CE
-           inherit! CE as x
-         *)
-     (* ce can be a class body and is therefore tagged.
-      *)
-     tag_class_expr write_tag ce
-  | Pcf_val({txt; loc}, _, _) -> write_tag loc txt
-        (* val x = E
-           val virtual x: T
-         *)
-  | Pcf_method({txt; loc}, _, _) -> write_tag loc txt
-        (* method x = E            (E can be a Pexp_poly)
-           method virtual x: T     (T can be a Ptyp_poly)
-         *)
+let rec tag_class_field_def write_tag cfd =
+  match [@flushcatchall] cfd.pcf_desc with
+    | Pcf_inherit(_, ce, _) ->
+	  (* inherit CE
+	     inherit CE as x
+	     inherit! CE
+	     inherit! CE as x
+	   *)
+       (* ce can be a class body and is therefore tagged.
+	*)
+       tag_class_expr write_tag ce
+    | Pcf_val({txt; loc}, _, _) -> write_tag loc txt
+	  (* val x = E
+	     val virtual x: T
+	   *)
+    | Pcf_method({txt; loc}, _, _) -> write_tag loc txt
+	  (* method x = E            (E can be a Pexp_poly)
+	     method virtual x: T     (T can be a Ptyp_poly)
+	   *)
 
-  | Pcf_constraint _
-        (* constraint T1 = T2 *)
-  | Pcf_initializer _
-        (* initializer E *)
-  | Pcf_attribute _
-        (* [@@@id] *)
-  | Pcf_extension _
-        (* [%%id] *)
-    -> ()
+    | Pcf_constraint _
+	  (* constraint T1 = T2 *)
+    | Pcf_initializer _
+	  (* initializer E *)
+    | Pcf_attribute _
+	  (* [@@@id] *)
+    | Pcf_extension _
+	  (* [%%id] *)
+    | _ -> ()
 
-and tag_class_expr write_tag ce = match ce.pcl_desc with
+and tag_class_expr write_tag ce = match [@flushcatchall] ce.pcl_desc with
   | Pcl_structure {pcstr_fields; _ } ->
         (* object ... end *)
      List.iter (tag_class_field_def write_tag) pcstr_fields
@@ -241,133 +258,137 @@ and tag_class_expr write_tag ce = match ce.pcl_desc with
            ['a1, ..., 'an] c *)
   | Pcl_extension _
         (* [%id] *)
-    -> ()
+  | _ -> ()
 
 let tag_class_def write_tag {pci_name = {txt; loc}; pci_expr; _} =
     write_tag loc txt;
     tag_class_expr write_tag pci_expr
 					      
 
-let rec tag_let_pattern write_tag pa = match pa.ppat_desc with
-  | Ppat_var { txt; loc } -> write_tag loc txt
-        (* x *)
-  | Ppat_alias(pat, {txt; loc}) ->
-        (* P as 'a *)
-     write_tag loc txt;
-     tag_let_pattern write_tag pat
-  | Ppat_tuple(pat_list) -> List.iter (tag_let_pattern write_tag) pat_list
-        (* (P1, ..., Pn)
-  
-           Invariant: n >= 2
-        *)
-  | Ppat_constraint(pat, _) -> tag_let_pattern write_tag pat
-        (* (P : T) *)
+let rec tag_let_pattern write_tag pa =
+  match [@flushcatchall] pa.ppat_desc with
+    | Ppat_var { txt; loc } -> write_tag loc txt
+	  (* x *)
+    | Ppat_alias(pat, {txt; loc}) ->
+	  (* P as 'a *)
+       write_tag loc txt;
+       tag_let_pattern write_tag pat
+    | Ppat_tuple(pat_list) -> List.iter (tag_let_pattern write_tag) pat_list
+	  (* (P1, ..., Pn)
 
-  | Ppat_any
-        (* _ *)
-  | Ppat_constant _
-        (* 1, 'a', "true", 1.0, 1l, 1L, 1n *)
-  | Ppat_interval _
-        (* 'a'..'z'
-  
-           Other forms of interval are recognized by the parser
-           but rejected by the type-checker. *)
-  | Ppat_construct _
-        (* C                None
-           C P              Some P
-           C (P1, ..., Pn)  Some (Ppat_tuple [P1; ...; Pn])
-         *)
-  | Ppat_variant _
-        (* `A             (None)
-           `A P           (Some P)
-         *)
-  | Ppat_record _
-        (* { l1=P1; ...; ln=Pn }     (flag = Closed)
-           { l1=P1; ...; ln=Pn; _}   (flag = Open)
-  
-           Invariant: n > 0
-         *)
-  | Ppat_array _
-        (* [| P1; ...; Pn |] *)
-  | Ppat_or _
-        (* P1 | P2 *)
-  | Ppat_type _
-        (* #tconst *) (* short-hand for all poly variants in tconst *)
-  | Ppat_lazy _
-        (* lazy P *)
-  | Ppat_unpack _
-        (* (module P)
-           Note: (module P : S) is represented as
-           Ppat_constraint(Ppat_unpack, Ptyp_package)
-         *)
-  | Ppat_exception _
-        (* exception P *)
-    (* Exception patterns are only permitted in proper matches with at 
-     * least 2 cases and not in let bindings. 
-     *)
-  | Ppat_extension _
-        (* [%id] *)
-    -> ()
-      
-let rec tag_str_item write_tag stri = match stri.pstr_desc with
-  | Pstr_value(_rec_flag, value_binding_list) ->
-        (* let P1 = E1 and ... and Pn = EN       (flag = Nonrecursive)
-           let rec P1 = E1 and ... and Pn = EN   (flag = Recursive)
-         *)
-     List.iter (fun {pvb_pat; _ } -> tag_let_pattern write_tag pvb_pat)
-	       value_binding_list     
-  | Pstr_primitive({pval_name = {txt; loc}; _}) -> write_tag loc txt
-        (*  val x: T
-            external x: T = "s1" ... "sn" *)
-    (* XXX the val x seems to be a copy/paste error *)
-  | Pstr_type(_rec, type_decl_list) ->
-        (* type t1 = ... and ... and tn = ... *)
-     List.iter (tag_type_decl write_tag) type_decl_list
-  | Pstr_typext({ptyext_path = {txt; loc}; ptyext_constructors; _ }) ->
-        (* type t1 += ... *)
-    (* 
-     * XXX check type A.B.t += X
-     * XXX try to rigger the Longident.last fatal exception
-     *)
-     write_tag loc (Longident.last txt);
-     List.iter (tag_extension_constuctor write_tag) ptyext_constructors
-  | Pstr_exception(ext_constr) ->
-        (* exception C of T
-           exception C = M.X *)
-     tag_extension_constuctor write_tag ext_constr
-  | Pstr_module(mod_binding) ->
-        (* module X = ME *)
-     tag_module_def_binding write_tag mod_binding
-  | Pstr_recmodule mod_bindings ->
-        (* module rec X1 = ME1 and ... and Xn = MEn *)
-     List.iter (tag_module_def_binding write_tag) mod_bindings
-  | Pstr_modtype mod_type_decl ->
-        (* module type S = MT *)
-     tag_module_type_decl write_tag mod_type_decl
-  | Pstr_class class_defs ->
-        (* class c1 = ... and ... and cn = ... *)
-     List.iter (tag_class_def write_tag) class_defs
-  | Pstr_class_type class_decls ->
-     (* class type ct1 = ... and ... and ctn = ... *)
-     List.iter (tag_class_decl write_tag) class_decls
+	     Invariant: n >= 2
+	  *)
+    | Ppat_constraint(pat, _) -> tag_let_pattern write_tag pat
+	  (* (P : T) *)
+    | Ppat_open(_mod_long_id, pat) -> tag_let_pattern write_tag pat
+          (* M.(P) *)
 
-  | Pstr_eval _
-        (* E *)
-  | Pstr_open _
-        (* open X *)
-  | Pstr_include _
-        (* include ME *)
-  | Pstr_attribute _
-        (* [@@@id] *)
-  | Pstr_extension _
-        (* [%%id] *)
-    -> ()
+    | Ppat_any
+	  (* _ *)
+    | Ppat_constant _
+	  (* 1, 'a', "true", 1.0, 1l, 1L, 1n *)
+    | Ppat_interval _
+	  (* 'a'..'z'
+
+	     Other forms of interval are recognized by the parser
+	     but rejected by the type-checker. *)
+    | Ppat_construct _
+	  (* C                None
+	     C P              Some P
+	     C (P1, ..., Pn)  Some (Ppat_tuple [P1; ...; Pn])
+	   *)
+    | Ppat_variant _
+	  (* `A             (None)
+	     `A P           (Some P)
+	   *)
+    | Ppat_record _
+	  (* { l1=P1; ...; ln=Pn }     (flag = Closed)
+	     { l1=P1; ...; ln=Pn; _}   (flag = Open)
+
+	     Invariant: n > 0
+	   *)
+    | Ppat_array _
+	  (* [| P1; ...; Pn |] *)
+    | Ppat_or _
+	  (* P1 | P2 *)
+    | Ppat_type _
+	  (* #tconst *) (* short-hand for all poly variants in tconst *)
+    | Ppat_lazy _
+	  (* lazy P *)
+    | Ppat_unpack _
+	  (* (module P)
+	     Note: (module P : S) is represented as
+	     Ppat_constraint(Ppat_unpack, Ptyp_package)
+	   *)
+    | Ppat_exception _
+	  (* exception P *)
+      (* Exception patterns are only permitted in proper matches with at 
+       * least 2 cases and not in let bindings. 
+       *)
+    | Ppat_extension _
+	  (* [%id] *)
+    | _ -> ()
+
+let rec tag_str_item write_tag stri =
+  match [@flushcatchall] stri.pstr_desc with
+    | Pstr_value(_rec_flag, value_binding_list) ->
+	  (* let P1 = E1 and ... and Pn = EN       (flag = Nonrecursive)
+	     let rec P1 = E1 and ... and Pn = EN   (flag = Recursive)
+	   *)
+       List.iter (fun {pvb_pat; _ } -> tag_let_pattern write_tag pvb_pat)
+		 value_binding_list     
+    | Pstr_primitive({pval_name = {txt; loc}; _}) -> write_tag loc txt
+	  (*  val x: T
+	      external x: T = "s1" ... "sn" *)
+      (* XXX the val x seems to be a copy/paste error *)
+    | Pstr_type(_rec, type_decl_list) ->
+	  (* type t1 = ... and ... and tn = ... *)
+       List.iter (tag_type_decl write_tag) type_decl_list
+    | Pstr_typext({ptyext_path = {txt; loc}; ptyext_constructors; _ }) ->
+	  (* type t1 += ... *)
+      (* 
+       * XXX check type A.B.t += X
+       * XXX try to rigger the Longident.last fatal exception
+       *)
+       write_tag loc (Longident.last txt);
+       List.iter (tag_extension_constuctor write_tag) ptyext_constructors
+    | Pstr_exception(ext_constr) ->
+	  (* exception C of T
+	     exception C = M.X *)
+       tag_extension_constuctor write_tag ext_constr
+    | Pstr_module(mod_binding) ->
+	  (* module X = ME *)
+       tag_module_def_binding write_tag mod_binding
+    | Pstr_recmodule mod_bindings ->
+	  (* module rec X1 = ME1 and ... and Xn = MEn *)
+       List.iter (tag_module_def_binding write_tag) mod_bindings
+    | Pstr_modtype mod_type_decl ->
+	  (* module type S = MT *)
+       tag_module_type_decl write_tag mod_type_decl
+    | Pstr_class class_defs ->
+	  (* class c1 = ... and ... and cn = ... *)
+       List.iter (tag_class_def write_tag) class_defs
+    | Pstr_class_type class_decls ->
+       (* class type ct1 = ... and ... and ctn = ... *)
+       List.iter (tag_class_decl write_tag) class_decls
+
+    | Pstr_eval _
+	  (* E *)
+    | Pstr_open _
+	  (* open X *)
+    | Pstr_include _
+	  (* include ME *)
+    | Pstr_attribute _
+	  (* [@@@id] *)
+    | Pstr_extension _
+	  (* [%%id] *)
+    | _ -> ()
 
 and tag_module_def_binding write_tag {pmb_name = {txt; loc}; pmb_expr; _ } =
   write_tag loc txt;
   tag_module_expr write_tag pmb_expr
 
-and tag_module_expr write_tag me = match me.pmod_desc with
+and tag_module_expr write_tag me = match [@flushcatchall] me.pmod_desc with
   | Pmod_structure str_items ->
         (* struct ... end *)
      List.iter
@@ -390,13 +411,13 @@ and tag_module_expr write_tag me = match me.pmod_desc with
         (* (val E) *)
   | Pmod_extension _
         (* [%id] *)
-    -> ()
+  | _ -> ()
 
 (* Generate tags for unit_ast of file file by calling write_tag 
  * for each tag.
  *)
 let generate_tags write_tag unit_ast =
-  (match unit_ast with
+  (match [@flushcatchall] unit_ast with
     | Sig_ast sig_items ->
       List.iter
 	(tag_sig_item write_tag)
@@ -405,6 +426,7 @@ let generate_tags write_tag unit_ast =
       List.iter
 	(tag_str_item write_tag)
 	str_items
+    | _ -> ()
   )
 
 
